<?php

namespace Vtb\VtbPay\Gateway\Http\Client\Api;

use Vtb\VtbPay\Gateway\Http\Client\Common\Request,
    Vtb\VtbPay\Gateway\Http\Client\Exception\VtbPayException,
    Symfony\Component\Dotenv\Dotenv;

/**
 * Класс VtbApi предоставляет методы для взаимодействия с API VTB.
 */
class VtbApi extends Request
{
    private string $mode;
    private string $clientMerchant;
    private array $clientData;
    private static array $modeUrls;


    /**
     * Конструктор класса VtbApi.
     *
     * @param string $clientId     Идентификатор клиента.
     * @param string $clientSecret Секрет клиента.
     * @param bool   $test         Режим тестирования (по умолчанию - false).
     * @param string $merchant     Идентификатор продавца (по умолчанию - пустая строка).
     *
     * @throws VtbPayException Если не заполнены обязательные поля $clientId или $clientSecret.
     */
    public function __construct(
        string $clientId,
        string $clientSecret,
        bool $test = false,
        string $merchant = ''
    ) {
        if (empty($clientId) || empty($clientSecret)) {
            throw new VtbPayException('Обязательные поля не заполнены.');
        }

        $this->clientData = [
            'client_id' => trim($clientId),
            'client_secret' => trim($clientSecret),
            'grant_type' => 'client_credentials'
        ];

        $this->clientMerchant = trim($merchant);
        $this->mode = $test ? 'TEST' : 'PROD';

        $dotenv = new Dotenv(); // Получаем настройки из .env в корне
        $dotenv->load(dirname(__DIR__, 4) . '/.env');
        self::$modeUrls = [
            'PROD' => [
                'TOKEN' => $_ENV['PROD_TOKEN'],
                'ORDER' => $_ENV['PROD_ORDER'],
                'ORDER-PREAUTH' => $_ENV['PROD_ORDER_PREAUTH'],
                'REFUND' => $_ENV['PROD_REFUND'],
            ],
            'TEST' => [
                'TOKEN' => $_ENV['TEST_TOKEN'],
                'ORDER' => $_ENV['TEST_ORDER'],
                'ORDER-PREAUTH' => $_ENV['TEST_ORDER_PREAUTH'],
                'REFUND' => $_ENV['TEST_REFUND'],
            ]
        ];
    }


    /**
     * Получает URL для заданного типа запроса.
     *
     * @param string $type Тип запроса ('TOKEN', 'ORDER' или 'REFUND').
     *
     * @return string URL для запроса.
     */
    private function getUrl(string $type): string
    {
        return self::$modeUrls[$this->mode][$type] ?? '';
    }


    /**
     * Генерирует массив заголовков для запроса.
     *
     * @param array  $extraHeaders    Дополнительные заголовки.
     * @param bool   $authorization   Флаг авторизации (по умолчанию - false).
     *
     * @return array Массив заголовков для запроса.
     */
    private function getHeaders(array $extraHeaders = [], bool $authorization = false): array
    {
        $headers = $extraHeaders;

        if ($authorization) {
            $headers = array_merge($this->getAuthorizationHeaders(), $headers);
        }

        if (!empty($this->clientMerchant)) {
            $headers[] = 'Merchant-Authorization: ' . $this->clientMerchant;
        }

        return $headers;
    }


    /**
     * Получает заголовки авторизации.
     *
     * @return array Массив заголовков для авторизации.
     *
     * @throws VtbPayException Если не удается получить токен доступа.
     */
    private function getAuthorizationHeaders(): array
    {
        $token = $this->getAccessToken();

        if (!isset($token['access_token']) || !isset($token['token_type'])) {
            throw new VtbPayException('Ошибка при получении токена доступа.', [
                'token' => $token
            ]);
        }

        return [
            'X-IBM-Client-Id: ' . explode('@', $this->clientData['client_id'])[0],
            'Authorization: ' . $token['token_type'] . ' ' . $token['access_token'],
        ];
    }


    /**
     * Получает массив с данными токена доступа.
     *
     * @return array Массив с токеном доступа.
     */
    private function getAccessToken(): array
    {
        $response = $this->sendRequest(
            $this->getUrl('TOKEN'),
            $this->clientData,
            $this->getHeaders([
                'Content-Type: application/x-www-form-urlencoded'
            ])
        );

        return $response;
    }


    /**
     * Получает сгенерированный данные о странице оплаты заказа.
     *
     * @param string $orderId             Идентификатор заказа.
     * @param string $email               Email покупателя.
     * @param int    $orderCreate         Дата создания заказа.
     * @param float  $amount              Сумма заказа.
     * @param string $returnUrl           URL для возврата после оплаты.
     * @param bool  $twoStage            Двухстадийный платеж.
     * @param array  $items               Массив товаров заказа.
     * @param string $appendedOrderName   Добавочный текст к названию заказа.
     *
     * @return array Результат запроса.
     * @throws VtbPayException Если не удается получить ссылки для оплаты.
     */
    public function getOrderLink(
        string $orderId,
        string $email,
        int $orderCreate,
        float $amount,
        string $returnUrl,
        bool $twoStage = false,
        array $items = [],
        string $appendedOrderName = ''
    ): array {
        $headers = $this->getHeaders(['Content-Type: application/json'], true);
        $timestamp = $orderCreate + 86400; // +1 day
        $expire = date('Y-m-d\TH:i:s.v\Z', $timestamp); // ISO 8601

        $body = [
            'orderId' => $orderId,
            'orderName' => 'Заказ #' . $orderId  . ($appendedOrderName ? '. ' . $appendedOrderName : ''),
            'customer' => ['email' => $email],
            'amount' => ['value' => $amount, 'code' => 'RUB'],
            'expire' => $expire,
            'returnUrl' => $returnUrl
        ];

        if (!empty($items)) {
            $body['bundle'] = [
                'fiscalInfo' => [
                    'clientEmail' => $email
                ],
                'items' => $items
            ];
        }

        $response = $this->sendRequest(
            $this->getUrl($twoStage ? 'ORDER-PREAUTH' : 'ORDER'),
            $body,
            $headers
        );

        if (
            !isset($response['object']['payUrl']) ||
            empty($response['object']['payUrl'])
        ) {
            throw new VtbPayException('Ошибка при получении ссылки для оплаты.', [
                'response' => $response
            ]);
        }

        return $response;
    }


    /**
     * Получает информацию о заказе.
     *
     * @param string $orderId Идентификатор заказа.
     *
     * @return array Результат запроса.
     * @throws VtbPayException Если не удается получить статус платежа.
     */
    public function getOrderInfo(string $orderId): array
    {
        $headers = $this->getHeaders([], true);

        $response = $this->sendRequest(
            $this->getUrl('ORDER') . '/' . $orderId,
            [],
            $headers,
            'GET'
        );

        if (
            !isset($response['object']['status']['value']) ||
            empty($response['object']['status']['value'])
        ) {
            throw new VtbPayException('Информация о статусе платежа отсутствует.', [
                'response' => $response
            ]);
        }

        return $response;
    }


    /**
     * Устанавливает возврат средств.
     *
     * @param string $orderId Идентификатор заказа.
     * @param float  $amount              Сумма заказа.
     * @param string $email               Email покупателя.
     * @param array  $items               Массив товаров заказа.
     *
     * @return array Результат запроса.
     * @throws VtbPayException Если нет информации об платеже
     */
    public function setRefunds(
        string $orderId,
        float $amount = 0,
        string $email = '',
        array $items = []
    ): array
    {
        $headers = $this->getHeaders(['Content-Type: application/json'], true);
        $response = $this->getOrderInfo($orderId);
        $transactionPayment = $response['object']['transactions']['payments'][0]['object'] ?? '';

        if (empty($transactionPayment)) {
            throw new VtbPayException('Информация о платеже отсутствует.', [
                'response' => $response
            ]);
        }

        if ($amount > $transactionPayment['amount']['value']) {
            throw new VtbPayException('Сумма возврата превышает сумму оплаты.', [
                'response' => $response
            ]);
        }

        $body = [
            'refundId' => $orderId . '-REFUND-' . time(),
            'paymentId' => $transactionPayment['paymentId'],
            'amount' => [
                'value' => $amount ?: $transactionPayment['amount']['value'],
                'code' => $transactionPayment['amount']['code']
            ]
        ];

        if (!empty($items) && !empty($email)) {
            $body['bundle'] = [
                'fiscalInfo' => [
                    'clientEmail' => $email
                ],
                'items' => $items
            ];
        }

        $response = $this->sendRequest(
            $this->getUrl('REFUND'),
            $body,
            $headers
        );

        if (isset($response['error']) && !empty($error = $response['error'])) {
            throw new VtbPayException(
                is_array($error) ? $error['description'] ?? 'Ошибка возврата средств.' : $error, [
                'response' => $response
            ]);
        }

        return $response;
    }
}
